from plone import api
from zope import interface
from zope.interface import directlyProvides
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from zope.interface import provider
from plone.app.vocabularies.terms import safe_simplevocabulary_from_values


LEVEL_TYPES = SimpleVocabulary(
    [SimpleTerm(value=u'Beginner', title=u'Beginner'),
     SimpleTerm(value=u'Intermediate', title=u'Intermediate'),
     SimpleTerm(value=u'Expert', title=u'Expert')]
    )

AUDIENCE_TYPES = SimpleVocabulary(
    [SimpleTerm(value=u'User', title=u'User'),
     SimpleTerm(value=u'Integrator', title=u'Integrator'),
     SimpleTerm(value=u'Designer', title=u'Designer'),
     SimpleTerm(value=u'Developer', title=u'Developer')]
    )

def persons(context):
    catalog = api.portal.get_tool('portal_catalog')
    person_brains = catalog(portal_type=('person', 'keynoter'),
                        sort_on='sortable_title',
                        sort_order='ascending')
    terms = []
    for brain in person_brains:
        token = brain.getPath()
        terms.append(SimpleTerm(
            value=brain.UID,
            token=token,
            title=brain.Title
            ))
    return SimpleVocabulary(terms)
directlyProvides(persons, IContextSourceBinder)

AUDIENCETYPE_VOCAB = {
    'id': 'audience-options'
}
LEVELTYPE_VOCAB = {
    'id': 'level-type-options'
}
PERSON_VOCAB = {
    'id': 'person-options'
}


@interface.implementer(IVocabularyFactory)
class AudiuenceVocab(object):
    """Vocabulary for Audience Type
    """

    def __init__(self, vocab_id=''):
        self.vocab_id = vocab_id

    def __call__(self, context):
        """ Make the vocabulary
        """
        return AUDIENCE_TYPES


@interface.implementer(IVocabularyFactory)
class LevelTypeVocab(object):
    """Vocabulary for Talk Level
    """

    def __init__(self, vocab_id=''):
        self.vocab_id = vocab_id

    def __call__(self, context):
        """ Make the vocabulary
        """
        return LEVEL_TYPES


@interface.implementer(IVocabularyFactory)
class PersonVocab(object):
    """Vocabulary for Person
    """

    def __init__(self, vocab_id=''):
        self.vocab_id = vocab_id

    def __call__(self, context):
        """ Make the vocabulary
        """
        return persons(context)

@provider(IVocabularyFactory)
def TracksVocabularyFactory(context):
    values = api.portal.get_registry_record('conferencepolicy.tracks')
    return safe_simplevocabulary_from_values(values)

@provider(IVocabularyFactory)
def TalkDurationVocabularyFactory(context):
    values = api.portal.get_registry_record(
                'conferencepolicy.talk_durations')
    return safe_simplevocabulary_from_values(values)


@provider(IVocabularyFactory)
def TrainingDurationVocabularyFactory(context):
    values = api.portal.get_registry_record(
                'conferencepolicy.training_durations')
    return safe_simplevocabulary_from_values(values)

audience = AudiuenceVocab(AUDIENCETYPE_VOCAB['id'])
level_types = LevelTypeVocab(LEVELTYPE_VOCAB['id'])
speakers = PersonVocab(PERSON_VOCAB['id'])
